local E = AleaUI
local OT = E:Module("ObjectivesTracker")
local Skins = E:Module("Skins")

local status = false

local WatchFrameHolder = CreateFrame("Frame", "WatchFrameHolder", E.UIParent)
WatchFrameHolder:SetWidth(130)
WatchFrameHolder:SetHeight(22)
WatchFrameHolder:Show()

local WatchFrameUpdater = CreateFrame("Frame", "WatchFrameUpdater", E.UIParent)
WatchFrameUpdater:Show()
WatchFrameUpdater:SetScript('OnUpdate', function(self, elapsed)
	self.elapsed = ( self.elapsed or 0 ) + elapsed
	if self.elapsed < 1 then return end
	self.elapsed = 0
	local show = true
	
	for i=1, 5 do
		local unit = 'boss'..i		
		if UnitExists(unit) then
			WatchFrameHolder:Hide()
			show = false
			break
		end
	end
	
	if show and not WatchFrameHolder:IsShown() then
		WatchFrameHolder:Show()
	end
end)

local header = { 
	["QuestHeader"] = true,
	["AchievementHeader"] = true,
	["ScenarioHeader"] = true,
}

local function Load()
	E:Mover(WatchFrameHolder, "watchFrameMover")
	
	ObjectiveTrackerFrame:SetIgnoreFramePositionManager(true)
	ObjectiveTrackerFrame:SetParent(WatchFrameHolder)
	ObjectiveTrackerFrame:ClearAllPoints()
	ObjectiveTrackerFrame:SetPoint('TOPRIGHT', WatchFrameHolder, 'TOPRIGHT')
	ObjectiveTrackerFrame:SetHeight(500)
	ObjectiveTrackerFrame:SetClampedToScreen(true)
	ObjectiveTrackerFrame:SetResizable(true)
	--[==[
	local sizing = CreateFrame("Frame", nil, ObjectiveTrackerFrame)
	sizing:SetSize(18, 18)
	sizing.isMouseDown = false
	sizing:RegisterForDrag("LeftButton")
	sizing:SetPoint("BOTTOMLEFT", ObjectiveTrackerFrame, "BOTTOMLEFT", -1, 1)
	sizing:SetScript("OnDragStart", function(self)
		self:GetParent():StartSizing() 
	end)
	sizing:SetScript("OnDragStop", function(self)
		self:GetParent():StopMovingOrSizing()
		self.isMouseDown = false
		self.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up") --0.4, 0.4, 0.4, 1)
	end)
	sizing:SetScript("OnEnter", function(self)
		self.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Highlight") --0.4, 0.4, 0.4, 0.8)
	end)
	sizing:SetScript("OnLeave", function(self)
		if not self.isMouseDown then
			self.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up") --0.4, 0.4, 0.4, 0.4)
		end
	end)
	sizing:SetScript("OnMouseUp", function(self)
		self.isMouseDown = false
		self.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up") --0.4, 0.4, 0.4, 1)
	end)
	sizing:SetScript("OnMouseDown", function(self)
		self.isMouseDown = true
		self.bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Down") --0.4, 0.4, 0.4, 0.8)
	end)
	
	local sizing_bg = sizing:CreateTexture(nil, "BACKGROUND", nil, 1)
	sizing_bg:SetAllPoints()
	sizing_bg:SetTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up") --0.4, 0.4, 0.4, 0.4)	
	sizing.bg = sizing_bg
	]==]
	ObjectiveTrackerFrame.HeaderMenu.Title:SetFont(E.media.default_font, 12,'NONE')
	ObjectiveTrackerFrame.HeaderMenu.Title:SetTextColor(1, 1, 1, 1)
	for frame in pairs(header) do
		ObjectiveTrackerBlocksFrame[frame]:StripTextures("ARTWORK")
		ObjectiveTrackerBlocksFrame[frame].Text:SetWordWrap(true)
		ObjectiveTrackerBlocksFrame[frame].Text:SetFont(E.media.default_font, 12,'NONE')
		ObjectiveTrackerBlocksFrame[frame].Text:SetTextColor(1, 1, 1, 1)
	end
	
	BONUS_OBJECTIVE_TRACKER_MODULE.Header:StripTextures("ARTWORK")
	BONUS_OBJECTIVE_TRACKER_MODULE.Header.Text:SetWordWrap(true)
	BONUS_OBJECTIVE_TRACKER_MODULE.Header.Text:SetFont(E.media.default_font2, 12)
	
	hooksecurefunc(ObjectiveTrackerFrame,"SetPoint",function(_,_,parent)
		if parent ~= WatchFrameHolder then
			ObjectiveTrackerFrame:ClearAllPoints()
			ObjectiveTrackerFrame:SetPoint('TOPRIGHT', WatchFrameHolder, 'TOPRIGHT')
		end
	end)
	hooksecurefunc(ObjectiveTrackerFrame,"SetParent",function(_,parent)
		if parent ~= WatchFrameHolder then
			ObjectiveTrackerFrame:SetParent(WatchFrameHolder)
		end
	end)
	
	hooksecurefunc('BonusObjectiveTracker_AnimateReward', function(block)
		local rewardsFrame = ObjectiveTrackerBonusRewardsFrame;
	--	rewardsFrame:StripTextures()
		for i=1, #rewardsFrame.Rewards do
			
			local reward = rewardsFrame.Rewards[i];
		
			reward.Count:SetFont(E.media.default_font2, 12, 'OUTLINE')
			reward.Label:SetFont(E.media.default_font2, 12, 'OUTLINE')
			reward.ItemIcon:SetTexCoord(unpack(E.media.texCoord))
			
		end
	end)
	
	hooksecurefunc('ScenarioObjectiveTracker_AnimateReward', function()
		local rewardsFrame = ObjectiveTrackerScenarioRewardsFrame;
	--	rewardsFrame:StripTextures()
		for i=1, #rewardsFrame.Rewards do
			
			local reward = rewardsFrame.Rewards[i];
		
			reward.Count:SetFont(E.media.default_font2, 12, 'OUTLINE')
			reward.Label:SetFont(E.media.default_font2, 12, 'OUTLINE')
			reward.ItemIcon:SetTexCoord(unpack(E.media.texCoord))
			
		end
	end)
	
	local colorGradient = { '|cFFFF0000', '|cFFFF7a00', '|cFFFFFF00', '|cFF00FF00' }
	
	local stringHooked = {}

	local function HandlerForObjectiveString(self, text)
		if stringHooked[self] ~= text then
			stringHooked[self] = text
		
		--	print('Update text', text)
			
			local a1, a2, a3, a4 = string.find(text, '(%d+)/(%d+)')
			
			if a3 and a4 then
				local currentValue = tonumber(a3)
				local maxValue = tonumber(a4)
				
				if currentValue and maxValue then
					local percent = currentValue/maxValue
					
					local color = colorGradient[4]
					
					if percent < 0.4 then
						color = colorGradient[1]
					elseif percent < 0.7 then
						color = colorGradient[2]
					elseif percent < 1 then
						color = colorGradient[3]
				--	elseif percent < 0.8 then
				--		color = colorGradient[4]
					end
					
					local pattern = format('%d/%d', currentValue, maxValue)
				
					local tempText = text:gsub(pattern, color..pattern..'|r')
					
					stringHooked[self] = tempText
					
					self:SetText(tempText)
					
				--	print('T', percent, currentValue, maxValue, color, pattern, text)
				end
			end
		end
	end
	
	
	local function QuestItemButtonSkin(self)
	
		self.HotKey:SetFont(E.media.default_font2, 18, 'OUTLINE')
		self.HotKey:SetText('x')
		
		self:SetNormalTexture(nil)
		self:SetPushedTexture(nil)
		
		self.icon:SetTexCoord(0.07, 0.93,  0.07, 0.93)
		self.icon:SetDrawLayer("OVERLAY")
		
		local texture = self:CreateTexture()
		texture:SetAllPoints(self)
		texture:SetDrawLayer("OVERLAY")
		texture:SetColorTexture(1, 1, 1, 0.3)
		
		self:SetHighlightTexture(texture)
		
		local bg = self:CreateTexture()
		bg:SetOutside(self)
		bg:SetDrawLayer("BACKGROUND")
		bg:SetColorTexture(0, 0, 0, 1)
	end
	
	hooksecurefunc('QuestObjectiveItem_OnShow', QuestItemButtonSkin)
	hooksecurefunc('QuestObjectiveItem_OnHide', QuestItemButtonSkin)
	
	local colorWhite = 1
	local colorWhiteHighlight = 1.3
	local classR, classG, classB = RAID_CLASS_COLORS[E.myclass].r, RAID_CLASS_COLORS[E.myclass].g, RAID_CLASS_COLORS[E.myclass].b
	local classRH, classGH, classBH = RAID_CLASS_COLORS[E.myclass].r, RAID_CLASS_COLORS[E.myclass].g, RAID_CLASS_COLORS[E.myclass].b
	
	if E.myclass == 'PRIEST' then
		colorWhite = 0.7
		colorWhiteHighlight = 1
		
		classR = 120/255
		classG = 153/255
		classB = 243/255
		
		classRH, classGH, classBH = classR, classG, classB
	elseif E.myclass == 'SHAMAN' then
		colorWhite = 1.3
		colorWhiteHighlight = 1.6
	end
	--[==[
	OBJECTIVE_TRACKER_COLOR["Header"].r = classR*colorWhite
	OBJECTIVE_TRACKER_COLOR["Header"].g = classG*colorWhite
	OBJECTIVE_TRACKER_COLOR["Header"].b = classB*colorWhite
	
	OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].r = classRH*colorWhiteHighlight
	OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].g = classGH*colorWhiteHighlight
	OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].b = classBH*colorWhiteHighlight
	]==]
	
	local customColorText = { Header = {}, HeaderHighlight = {} }
	
	customColorText["Header"].r = classR*colorWhite
	customColorText["Header"].g = classG*colorWhite
	customColorText["Header"].b = classB*colorWhite
	
	customColorText["HeaderHighlight"].r = classRH*colorWhiteHighlight
	customColorText["HeaderHighlight"].g = classGH*colorWhiteHighlight
	customColorText["HeaderHighlight"].b = classBH*colorWhiteHighlight
	
	local function UpdateCustomTextColor(self, r,g,b,a)
		if OBJECTIVE_TRACKER_COLOR["Header"].r == r and
			OBJECTIVE_TRACKER_COLOR["Header"].g == g and
			OBJECTIVE_TRACKER_COLOR["Header"].b == b then
			
			self:SetTextColor(customColorText["Header"].r, customColorText["Header"].g,customColorText["Header"].b)
		elseif OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].r == r and
			OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].g == g and
			OBJECTIVE_TRACKER_COLOR["HeaderHighlight"].b == b then
			
			self:SetTextColor(customColorText["HeaderHighlight"].r, customColorText["HeaderHighlight"].g,customColorText["HeaderHighlight"].b)
		end
	end
	
	hooksecurefunc(DEFAULT_OBJECTIVE_TRACKER_MODULE, 'SetStringText', function(self, fontString, text, useFullHeight, colorStyle, useHighlight)			
		if not stringHooked[fontString] then
			stringHooked[fontString] = true
			
			if colorStyle == OBJECTIVE_TRACKER_COLOR["Header"] then
				fontString:SetFont(E.media.default_font2, 14, 'NONE')
				
				UpdateCustomTextColor(fontString, OBJECTIVE_TRACKER_COLOR["Header"].r, OBJECTIVE_TRACKER_COLOR["Header"].g, OBJECTIVE_TRACKER_COLOR["Header"].b)
				hooksecurefunc(fontString, 'SetTextColor', UpdateCustomTextColor)	
			else
				fontString:SetFont(E.media.default_font2, 12, 'NONE')
				
				hooksecurefunc(fontString, 'SetText', HandlerForObjectiveString)	
			end
		end
	end)

	hooksecurefunc(QUEST_TRACKER_MODULE, 'AddObjective', function(self, block, objectiveKey, text, lineType, useFullHeight, hideDash, colorStyle)
		
		local line = block.lines[objectiveKey];
		
		if line then	

		end
	end)
	
	local shineAnims = {}
	
	hooksecurefunc(SCENARIO_TRACKER_MODULE, 'AddProgressBar', function(self, block, line, criteriaIndex)
		local progressBar = self.usedProgressBars[block] and self.usedProgressBars[block][line]
		
	--	print('SCENARIO_TRACKER_MODULE', progressBar)
		
		if progressBar then
		
			if not progressBar.skinned then
				progressBar.skinned = true
				
				local statusBar = progressBar.Bar
				local IconBG 	= statusBar.IconBG
				local Icon		= statusBar.Icon
				local BarFrame 	= statusBar.BarFrame
				local BarFrame2 = statusBar.BarFrame2
				local BarFrame3 = statusBar.BarFrame3
				local BarBG		= statusBar.BarBG -- background
				
				statusBar:HookScript('OnValueChanged', function(self, value)
					if not shineAnims[self].ag:IsPlaying() then
						shineAnims[self].ag:Play()
					end
				end)
				
				local shine = statusBar:CreateTexture(nil, 'OVERLAY', nil, 1)
				shine:SetAllPoints()
				shine:SetColorTexture(1, 1, 1, 0.7)
				shine:SetAlpha(0)
	
				local ag = shine:CreateAnimationGroup()
				local a1 = ag:CreateAnimation("Alpha")
				a1:SetFromAlpha(0)
				a1:SetToAlpha(0.6)
				a1:SetDuration(0.3)
				a1:SetOrder(1)
				
				local a2 = ag:CreateAnimation("Alpha")
				a2:SetFromAlpha(0.6)
				a2:SetToAlpha(0)
				a2:SetDuration(0.5)
				a2:SetOrder(2)
				
				shine.ag = ag
				
				shineAnims[statusBar] = shine
				
				IconBG:SetAlpha(0)
				BarFrame:SetAlpha(0)
				
				statusBar:SetStatusBarTexture([[Interface\AddOns\AleaUI\media\Minimalist.tga]])
				BarBG:SetTexture([[Interface\AddOns\AleaUI\media\Minimalist.tga]])
				BarFrame2:Kill()
				BarFrame3:Kill()
				
				BarBG:SetVertexColor(32/255, 61/255, 116/255, 0.7)
				E:CreateBackdrop(statusBar, BarBG, { 0, 0, 0, 1 }, { 0, 0, 0, 0 }) --, background, size,step)				
				E:CreateBackdrop(progressBar, progressBar.Bar.Icon, { 0, 0, 0, 1 }, { 0, 0, 0, 0 }) --, background, size,step)
			
				statusBar.Starburst:Kill()
				statusBar.BarGlow:Kill()
				statusBar.Sheen:Kill()
				
				progressBar.Flare1:Kill()
				progressBar.Flare2:Kill()
				progressBar.SmallFlare1:Kill()
				progressBar.SmallFlare2:Kill()
				progressBar.FullBarFlare1:Kill()
				progressBar.FullBarFlare2:Kill()
			end
			
			progressBar.Bar.Icon:Hide();
			progressBar.Bar.IconBG:Hide();
			progressBar.Bar.Icon:SetUIBorderAlpha(0)
			
			if (not criteriaIndex) then
				local rewardQuestID = select(10, C_Scenario.GetStepInfo());

				if (rewardQuestID ~= 0) then
					-- reward icon; try the first item
					local _, texture = GetQuestLogRewardInfo(1, rewardQuestID);
					-- artifact xp
					local artifactXP, artifactCategory = GetQuestLogRewardArtifactXP(rewardQuestID);
					if ( not texture and artifactXP > 0 ) then
						local name, icon = C_ArtifactUI.GetArtifactXPRewardTargetInfo(artifactCategory);
						texture = icon or "Interface\\Icons\\INV_Misc_QuestionMark";
					end
					-- currency
					if ( not texture and GetNumQuestLogRewardCurrencies(rewardQuestID) > 0 ) then
						_, texture = GetQuestLogRewardCurrencyInfo(1, rewardQuestID);
					end
					-- money?
					if ( not texture and GetQuestLogRewardMoney(rewardQuestID) > 0 ) then
						texture = "Interface\\Icons\\inv_misc_coin_02";
					end
					-- xp
					if ( not texture and GetQuestLogRewardXP(rewardQuestID) > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL ) then
						texture = "Interface\\Icons\\xp_icon";
					end
					if ( texture ) then
						progressBar.Bar.Icon:SetTexture(texture);
						progressBar.Bar.Icon:Show();
						progressBar.Bar.IconBG:Show();
						progressBar.Bar.Icon:SetUIBorderAlpha(1)
						progressBar.Bar.BarGlow:SetAtlas("bonusobjectives-bar-glow-ring", true);
					end
				end
			end
		end
	end)
	
	hooksecurefunc(BONUS_OBJECTIVE_TRACKER_MODULE, 'AddProgressBar', function(self, block, line, questID, finished)
		local progressBar = self.usedProgressBars[block] and self.usedProgressBars[block][line]

		if progressBar then
		
			if not progressBar.skinned then
				progressBar.skinned = true
				
				local statusBar = progressBar.Bar
				local IconBG 	= statusBar.IconBG
				local Icon		= statusBar.Icon
				local BarFrame 	= statusBar.BarFrame
				local BarFrame2 = statusBar.BarFrame2
				local BarFrame3 = statusBar.BarFrame3
				local BarBG		= statusBar.BarBG -- background
				
				statusBar:HookScript('OnValueChanged', function(self, value)
					if not shineAnims[self].ag:IsPlaying() then
						shineAnims[self].ag:Play()
					end
				end)
				
				local shine = statusBar:CreateTexture(nil, 'OVERLAY', nil, 1)
				shine:SetAllPoints()
				shine:SetColorTexture(1, 1, 1, 0.7)
				shine:SetAlpha(0)
	
				local ag = shine:CreateAnimationGroup()
				local a1 = ag:CreateAnimation("Alpha")
				a1:SetFromAlpha(0)
				a1:SetToAlpha(0.6)
				a1:SetDuration(0.3)
				a1:SetOrder(1)
				
				local a2 = ag:CreateAnimation("Alpha")
				a2:SetFromAlpha(0.6)
				a2:SetToAlpha(0)
				a2:SetDuration(0.5)
				a2:SetOrder(2)
				
				shine.ag = ag
				
				shineAnims[statusBar] = shine
				
				
				IconBG:SetAlpha(0)
				BarFrame:SetAlpha(0)
				
				statusBar:SetStatusBarTexture([[Interface\AddOns\AleaUI\media\Minimalist.tga]])
				BarBG:SetTexture([[Interface\AddOns\AleaUI\media\Minimalist.tga]])
				BarFrame2:Kill()
				BarFrame3:Kill()
				
				BarBG:SetVertexColor(32/255, 61/255, 116/255, 0.7)
				E:CreateBackdrop(statusBar, BarBG, { 0, 0, 0, 1 }, { 0, 0, 0, 0 }) --, background, size,step)				
				E:CreateBackdrop(statusBar, progressBar.Bar.Icon, { 0, 0, 0, 1 }, { 0, 0, 0, 0 }) --, background, size,step)
			
				statusBar.Starburst:Kill()
				statusBar.BarGlow:Kill()
				statusBar.Sheen:Kill()
				
				progressBar.Flare1:Kill()
				progressBar.Flare2:Kill()
				progressBar.SmallFlare1:Kill()
				progressBar.SmallFlare2:Kill()
				progressBar.FullBarFlare1:Kill()
				progressBar.FullBarFlare2:Kill()
			end
			
			local _, texture = GetQuestLogRewardInfo(1, questID);
			-- next, currency
			if ( not texture and GetNumQuestLogRewardCurrencies(questID) > 0 ) then
				_, texture = GetQuestLogRewardCurrencyInfo(1, questID);
			end
			-- money?
			if ( not texture and GetQuestLogRewardMoney(questID) > 0 ) then
				texture = "Interface\\Icons\\inv_misc_coin_02";
			end
			-- xp
			if ( not texture and GetQuestLogRewardXP(questID) > 0 and UnitLevel("player") < MAX_PLAYER_LEVEL ) then
				texture = "Interface\\Icons\\xp_icon";
			end

			if not texture then			
				progressBar.Bar.Icon:SetUIBorderAlpha(0)
			else
				progressBar.Bar.Icon:SetUIBorderAlpha(1)
				progressBar.Bar.Icon:SetSize(17, 17)
				progressBar.Bar.Icon:SetMask(nil)
				progressBar.Bar.Icon:SetTexture(texture)
				progressBar.Bar.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
				progressBar.Bar.Icon:ClearAllPoints()
				progressBar.Bar.Icon:SetPoint("TOPLEFT", progressBar.Bar.BarBG, "TOPRIGHT", 3, 0)
			end			
		end
	end)	
end

E:OnInit2(Load)